;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; clojure 'context free art' Version 0.1
;
; This work is in the public domain
;
; Mike Messinides, 6Feb2008
;
; inspired by software at http://contextfreeart.org
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;See examples at the end of this file
;
;USAGE:
;
;(cfam pmap 
;  (gfn shape-name prob gfn-pmap [draw-command+] next-shape)
;   .
;   .
;   .
;)
;
;pmap is a map containing optional parameters (a default value is used if not specified):
; :title	frame title
; :max		maximum iterations of the main drawing sequence
; :startshape   first shape function called in the main drawing sequence
; :branchdepth  depth of recursive branch calls 
; :bmax		maximum iterations in each branch drawing sequence
; :x 		frame width
; :y 		frame height
; :scl-min	minimum scale factor (terminates drawing sequence)
;
;each gfn form defines a shape function associated with a shape name
;  shape-name	symbol
;  prob		probability factor ( >0 )
;  gfn-pmap     map of parameters (see below)
;  draw-command	any method call applicable to java.awt.Graphics2D
;  next-shape	selects the next shape to be drawn; if 'end', the drawing sequence is terminated 
;
;gfn-pmap contains the following optional parameters:
;  :x		translation x
;  :y		translation y
;  :shx		shear x
;  :shy		shear y
;  :rot		rotation
;  :sx		scaling x
;  :sy		scaling y
;  :branch      shape-name used to start a branch drawing sequence
;
;These parameters (except for :branch) are used to create an AffineTransform. 
;The transform is applied after all drawing commands are executed. If :branch 
;  
;When there is more than one shape function per shape name, a shape function
;is selected at random, with prob used as a weighting factor (not a straight probability) for the selection.
;The probability that a shape function is selected is (/ prob sum-prob) where
;sum-prob is the sum of all probability factors for the shape functions associated with
;the shape name. 
;

(defmacro COMMENT [& body]
         
)


(import '(javax.swing JFrame JLabel JTextField JButton) 
        '(java.awt.event ActionListener) 
        '(java.awt GridLayout Graphics2D RenderingHints Dimension)) 
 
(import '(cljext.swing IPainter DelegatingPanel))

(def *gprob*)
(def *gfn*)

(defn remwrap [v d] 
  (let [r (rem v d)]
    (if (< r 0) (+ r d) r)))

(defn alpha [val] (let [m (remwrap val 255)] (if m (if (= m 0) 255 m) 255)))
(defn nil0 [val] (if val val 0))
(defn nil1 [val] (if val val 1))
(defn trans [x y] (. java.awt.geom.AffineTransform (getTranslateInstance (nil0 x) (nil0 y))))
(defn scal [x y] (. java.awt.geom.AffineTransform (getScaleInstance (nil1 x) (nil1 y))))
(defn shear [x y] (. java.awt.geom.AffineTransform (getShearInstance (nil0 x) (nil0 y))))
(defn rot [deg] (. java.awt.geom.AffineTransform (getRotateInstance (. Math (toRadians (nil0 deg))))))
(defn color [r g b a] 
  (do
    ;(println (list r g b a));;;;DEBUG
    (new java.awt.Color 
         (remwrap (int (nil0 r)) 255)
         (remwrap (int (nil0 g)) 255)
         (remwrap (int (nil0 b)) 255)
         (alpha (int (nil0 a))))))

;from contextfreeart:
;"first translated ... then it is rotated ... then it is scaled ....then skewed and finally it is reflected"
;AffineTransform has no flip method, but scaling can be used:
;         :sx 1 :sy -1
;         :sx -1 :sy 1
;
(defn ax [pmap]
  (let [x  (:x pmap)      ;translation x
        y  (:y pmap)	;translation y
        shx (:shx pmap)	;shear x
        shy (:shy pmap)	;shear y
        rt (:rot pmap)	;rotation
        sx  (:sx pmap)	;scaling x
        sy  (:sy pmap)	;scaling y
        f (:f pmap)	;flip
        shearAT (shear shx shy)
        transAT (trans x y)
        scalAT (scal sx sy)
        rotAT (rot rt)]
        ;(doto shearAT (concatenate rotAT) (concatenate scalAT) (concatenate transAT))
        ;(doto shearAT (concatenate scalAT) (concatenate rotAT) (concatenate transAT))
    (doto shearAT (concatenate transAT) (concatenate rotAT) (concatenate scalAT))
    shearAT))

(defn minscale [pmap]
  (let [sx (:sx pmap)
        sy (:sy pmap)]
    (if sx (if sy (if (< sx sy) sx sy) sx) (if sy sy 1)))) 


(defn grx [#^java.awt.Graphics2D g2 pmap]
  (let [at (ax pmap)	;affine transform
        r (:r pmap)		;color red
        g (:g pmap)		;color green
        b (:b pmap)		;color blue
        a (:a pmap) 	;alpha
        prev-c (. g2 (getColor))
        prev-r (if prev-c (. prev-c (getRed)) 0)
        prev-g (if prev-c (. prev-c (getGreen)) 0)
        prev-b (if prev-c (. prev-c (getBlue)) 0)
        prev-a (if prev-c (. prev-c (getAlpha)) 255)
        c (color (+ (nil0 r) prev-r) (+ (nil0 g) prev-g) (+ (nil0 b) prev-b) (+ (if a a 255) prev-a) )]
        ;c (color r g b a)]   
    (. g2 (transform at))
    (. g2 (setColor c))))


(defmacro gr [g pmap & rest]
  `(do (doto ~g ~@rest)
       (grx ~g ~pmap)))



(defn canvas [title] 
  (let [frame (new javax.swing.JFrame title)
        pane (. frame (getContentPane))
        panel (new cljext.swing.DelegatingPanel)]
    (. frame (setDefaultCloseOperation (. javax.swing.JFrame DISPOSE_ON_CLOSE)))			
    (. panel (setMinimumSize (new java.awt.Dimension 100,100)) )
    (. pane (add panel)) 
    `(~frame ~panel)))  

;num fn is needed because of eql? problem with java.lang.Integer and clojure numbers
(defn num [x] (. clojure.lang.Num (from x))) 

;Populates *gprob* with probability lists for selecting the shape function given
;a shape name.
;The probability list is (sum-prob prob-vec) where
;prob-vec = [prob0 prob1 ...]
;sum-prob = prob0 + prob1 +...
;Returns the sequence number for the shape function; sequence number = index into the
; probability factor vector 
;The sequence number for the shape name is incremented on every call  
(defn make-gprob [gname prob]
  (let [pl (*gprob* gname)
        pl (if (not pl) (list 0 (vector)) pl)
        v  (second pl)
        sum-p (first pl)
        idx (num (count v))
        v (conj v prob)]
    (def *gprob* (assoc *gprob* gname (list (+ sum-p prob) v)))
    idx))

;
;selects the shape function given prob and the probability list stored in *gprob* 
;selects the first function (index 0) if prob <= 0
;selects the last function if prob >= 1
;
(defn select-gfn [prob p-list]
  (let [sum-prob (first p-list)
        p-vec (second p-list)
        pf (* prob sum-prob) ; scale the prob to fit the range of prob factors
        maxidx (- (count p-vec) 1)
        ;x (println pf)  ;;;;;;;;;;;;;;DEBUG
        ]
    (loop [idx 0 rsum (first p-vec)]
      (if (< pf rsum) idx
          (if (>= idx maxidx) maxidx (recur (+ 1 idx) (+ rsum (nth p-vec (+ idx 1)))))))))

(defn gfn-call [cmds g]
  (loop [cmds cmds]
    (if (first cmds) 
      (let [gfnlist (*gfn* (first cmds))
            at (apply (first gfnlist) (list g))]
        (recur (rest cmds))))))

(defn gfn-painter [cmds]
  (let [gfn-p
        (implement [IPainter]
          (paint [#^java.awt.Graphics2D g]
                 (. g (setRenderingHint (. RenderingHints KEY_ANTIALIASING), (. RenderingHints VALUE_ANTIALIAS_ON)))
                 (gfn-call cmds g)))]
    gfn-p))

(defn append [vec  body]
  (let [c0 (count vec)
        c1 (count body)
        max (- (+ c0 c1) 1)]
    (loop [i c0 r body v vec]
      (if (> i max) v
          (recur (+ 1 i) (rest r) (assoc v i (first r)))))))

(def *at-stack* (ref [])) ;pushes and pops may be executed in different threads

(defn g-push [#^java.awt.Graphics2D g]
  (sync nil
    (let [at (. g (getTransform))
          c  (. g (getColor))
          stack (deref *at-stack*)
          cnt (count stack)]
       ;(try
 ;       (println (list 'push cnt (.. Thread (currentThread) (getName))))
 ;       (println stack)
      (set *at-stack* (assoc stack cnt (list at c)))
        ;(catch Exception e (println (list 'exception at c cnt))))
      )))

(defn g-pop [#^java.awt.Graphics2D g]
  (sync nil
    (let [stack (deref *at-stack*)
          last-idx (- (count stack) 1)
          at-c (nth stack last-idx)
          nxt (take last-idx stack)]
 ;        (println (list 'pop last-idx (.. Thread (currentThread) (getName))))
      (. g (setTransform (first at-c)))
      (. g (setColor (second at-c)))
;         (set *at-stack* (subvec stack 0 last-idx))
      (set *at-stack*  (if nxt
                         (. clojure.lang.PersistentVector (create nxt))
                         []))
         ;(println (deref *at-stack*))
      )))


;;;;subvec bug
;(def v [1 2 3])
;(assoc v (count v) 4) ; ok [1 2 3 4]
;(def vsub (subvec v 0 1)) ; ok [1]
;(. vsub (getClass))
;(def v2 (assoc vsub (count vsub) 2)) ; =>IndexOutofBoundsException
;workaround - (. clojure.lang.PersistentVector (create (take (- (count v) 1) v)))


;;;;;;;;;;;;;;;

          
(def make-branch nil)         


(defn make-gfn-list [maxi startshape startscale branchdepth bmax scl-min] 
  (loop [fkey (list startshape 0) cmds (vector) cnt 0 scale startscale] 
       ;  (println fkey)  ;;;;;;DEBUG
    (let [fval (*gfn* fkey) nxt (second fval) gprob (*gprob* nxt) ]               
      (if (or (== cnt maxi) (not gprob) (< scale scl-min)) (conj cmds fkey)
          (do
                 ; (println nxt)  ;;;;;;;;DEBUG
                 ; (println (list cnt scale)) ;;;;;;;;;DEBUG
            (recur (list nxt (select-gfn (. Math (random)) gprob)) 
                        ;(conj cmds fkey) 
                   (append cmds (if (and (> branchdepth 0) (nth fval 3)) 
                                  (make-branch fkey bmax (nth fval 3) scale (- branchdepth 1) scl-min) 
                                  (list fkey)))
                   (+ cnt 1) 
                   (* scale (nth fval 2))))))))


(defn make-branch [cmd max startshape startscale branchdepth scl-min]
  (let [branch (make-gfn-list max startshape startscale branchdepth max scl-min)
        vec0 [cmd '(g-push)] 
        vec1 (append vec0 branch)]
    (assoc vec1 (count vec1) '(g-pop))))

(defmacro gfn [gname prob pmap & body]
  (let [nxt# (first (reverse body))
        cmd# (rest (reverse body))]
    `(let [n# (fn [#^java.awt.Graphics2D g#] (gr g# ~pmap ~@cmd#))
           i# (list '~gname (make-gprob '~gname ~prob)) 
           b# (:branch ~pmap) ]
       (do 
         (def *gfn* (assoc *gfn* i# (list n# '~nxt# (minscale ~pmap) b#)))
         i#))))



;;;;;;;;;;;;;;TEST START 
;(def *gfn* {})
;(def *gprob* {})
;(gfn a 1 {:x 20 :y 30} (drawLine 0,0,10,10) a)
;(gfn a 0.1 {:x 10 :y 3} (drawLine 0,0,1,1) b)
;(gfn b 1 {} (drawString "test" 0,0) b)
;(gfn b 0.1 {} (drawString "end" 0,0) end)
;(make-gfn-list 10)
;;;;;;;;;;;;;;TEST END


;;;;;;;;;;;;;;TEST START 
;(def *gfn* {})
;(def *gprob* {})
;(make-gfn a 0 {:x 20 :y 30} (drawLine 0,0,10,10) a)
;(make-gfn a 1 {:x 10 :y 3} (drawLine 0,0,1,1) b)

;;;;;;;;;;;;;;TEST END

  

(defn cfa [pmap]
  (let [maxi (if (:max pmap) (:max pmap) 1000)
        x (if (:x pmap) (:x pmap) 800)
        y (if (:y pmap) (:y pmap) 600)
        title (if (:title pmap) (:title pmap) "clojure cfa")
        startshape (if (:startshape pmap) (:startshape pmap) (first (keys *gfn*)))
        bdepth (if (:branchdepth pmap) (:branchdepth pmap) 5)
        bmax  (if (:bmax pmap) (:bmax pmap) 100)
        scl-min (if (:scl-min pmap) (:scl-min pmap) 0.01)
             

        c-list (canvas title)
        gfn-list (make-gfn-list maxi startshape 1 bdepth bmax scl-min)
        dims (new java.awt.Dimension x,y)
        painter (gfn-painter gfn-list)]

    (def *gfn* (assoc *gfn* '(g-push) (list g-push)))
    (def *gfn* (assoc *gfn* '(g-pop) (list g-pop)))
    (. (second c-list) (setPainter painter))
    (. (second c-list) (setMinimumSize dims))
    (doto (first c-list) (setSize dims) (setVisible true))))

(defmacro cfam [pmap & body]
  `(do
     (def *at-stack* (ref []))
     (def *gfn* {})
     (def *gprob* {})
     ~@body
     (cfa ~pmap)))

(defn test-rot [pmap]
  (let [c-list (canvas "test-rotation")
        painter
        (implement [IPainter]
          (paint [#^java.awt.Graphics2D g]
                 (. g (setRenderingHint (. RenderingHints KEY_ANTIALIASING), (. RenderingHints VALUE_ANTIALIAS_ON)))
                 (grx g pmap)
                 (. g (drawRect 0,0,10,10))))]
    (. (second c-list) (setPainter painter))
    (doto (first c-list) (setSize (new java.awt.Dimension 500,500)) (setVisible true))))

  
(COMMENT 

(cfam {:title "test" :max 20000 :x 800 :y 800 :startshape 'shape0}
      (gfn shape0 1 {:x 200 :y 200 }
           (drawRect 0,0,0,0)
           shape1)      
      (gfn shape1 0.5 {:x 2 :y 3 :r 10 :b 10 }
           (fillRect 0,0,4,4) shape2)
      (gfn shape1 0.5 {:x 2 :y 3 :r -10 }
           (drawLine 0,0,10,10)
           (drawRect 10,10,5,5)
           (fillRect 10,10,5,5) shape2)
      (gfn shape2 1 {:rot 12 :sx 0.999 :sy 0.999 }
           (drawArc 0,0,50,100,20,50)
           shape1)
      (gfn shape2 0.5 {:rot -12  }
           (drawArc 0,0,50,100,20,50)
           shape1))
)

(COMMENT 

(cfam {:title "hairball with chunks" :max 7000 :x 800 :y 600 :startshape 'shape0}
      (gfn shape0 1 {:x 200 :y 200 }
           (drawRect 0,0,0,0)
           shape1)      
      (gfn shape1 0.5 {:x 2 :y 3 :r 10 :b 10 }
           (fillRect 0,0,4,4) shape2)
      (gfn shape1 0.05 {:x 25 :y 20 }
           (drawRect 0,0,100,100) shape2)
      (gfn shape1 0.5 {:x 2 :y 3 :r -10 }
           (drawLine 0,0,10,10)
           (drawRect 10,10,5,5)
           (fillRect 10,10,5,5) shape2)
      (gfn shape2 1 {:rot 12 :sx 0.999 :sy 0.999 }
           (drawArc 0,0,50,100,20,50)
           shape1)
      (gfn shape2 0.5 {:rot -12  }
           (drawArc 0,0,50,100,20,50)
           shape1))
)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(COMMENT
      
(cfam {:title "pretty spiral thingy" :max 1000 :x 800 :y 800 :startshape 'shape1}
      (gfn shape1 1 {:x 400 :y 400 :g -100}
           (drawRect 0,0,0,0)
           shape2)
      (gfn shape2 1 {:rot -12 :sx 0.999 :sy 0.999 :r 3 :b -3}
           (drawArc 0,0,300,400,20,30)
           shape3)
      (gfn shape3 1 {:rot -12 :sx 0.999 :sy 0.999 }
           (drawArc 0,0,300,400,20,30)
           shape4)
      (gfn shape4 1 {:rot -12 :sx 0.999 :sy 0.999 }
           (drawArc 0,0,300,400,20,30)
           shape2)))
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(COMMENT

(cfam {:title "featherball" :max 350 :x 800 :y 800 :startshape 'shape0
      :branchdepth 3 :bmax 100}
      (gfn shape0 1 {:x 400 :y 400 }
           (drawRect 0,0,0,0)
           shape1)      
      (gfn shape1 0.5 {:x 2 :y 3 :r 10 :b 10 }
           (fillRect 0,0,4,4) shape2)
      (gfn shape1 0.5 {:x 2 :y 3 :r -10 }
           (drawLine 0,0,10,10)
           (drawRect 10,10,5,5)
           (fillRect 10,10,5,5) shape2)
      (gfn shape2 1 {:rot 12 :sx 0.999 :sy 0.999 }
           (drawArc 0,0,50,100,20,50)
           shape1)
      (gfn shape2 0.5 {:rot -12  }
           (drawArc 0,0,50,100,20,50)
           shape1)
      (gfn shape2 0.1 {:rot -12 :branch 'shape1 }
           (drawArc 0,0,50,100,20,50)
           shape1)))
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(COMMENT 

(cfam {:title "multi spiral thingy" :max 900 :x 800 :y 700 :startshape 'shape1}
       
      (gfn shape1 1 {:x 400 :y 350 :g -100}
           (drawRect 0,0,0,0)
           shape2)
      (gfn shape2 1 {:rot -12 :sx 0.999 :sy 0.999 :r 3 :b -3}
           (drawArc 0,0,300,400,20,30)
           shape3)
      (gfn shape3 1 {:rot -12 :sx 0.999 :sy 0.999 }
           (drawArc 0,0,300,400,20,30)
           shape4)
      (gfn shape4 1 {:rot -12 :sx 0.999 :sy 0.999 }
           (drawArc 0,0,300,400,20,30)
           shape2)
      (gfn shape4 0.02 {:rot -12 :x 90 :y 50 :sx 0.9 :sy 0.9 :branch 'shape1}
           (drawArc 0,0,300,400,20,30)
           shape2)))
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(COMMENT
 
(cfam {:title "chunky featherball" :max 2500 :x 800 :y 700 :startshape 'shape0
        :branchdepth 2 :bmax 100}
        (gfn shape0 1 {:x 100 :y 100 :sx 1.2 :sy 1.2}
             (drawRect 0,0,0,0)
             shape1)      
        (gfn shape1 0.5 {:x 2 :y 3 :r 10 :b 10 }
             (fillRect 0,0,4,4) shape2)
        (gfn shape1 0.05 {:x 25 :y 20 }
             (drawRect 0,0,100,100) shape2)
        (gfn shape1 0.5 {:x 2 :y 3 :r -10 }
             (drawLine 0,0,10,10)
             (drawRect 10,10,5,5)
             (fillRect 10,10,5,5) shape2)
        (gfn shape2 1 {:rot 12 :sx 0.999 :sy 0.999 }
             (drawArc 0,0,50,100,20,50)
             shape1)
        (gfn shape2 0.5 {:rot -12  }
             (drawArc 0,0,50,100,20,50)
             shape1)
        (gfn shape2 0.05 {:sx 0.9 :sy 0.9 :x 50 :branch 'shape2  }
             (drawArc 0,0,50,100,20,50)
             shape1)))
(COMMENT

(cfam {:title "arcs" :max 10000 :x 800 :y 700 :startshape 'shape0
      :branchdepth 2 :bmax 100}
      (gfn shape0 1 {:x 400 :y 200 :rot 45}
           (drawRect 0,0,0,0)
           shape1)      
      (gfn shape1 1 {:x 1 :y 1 :rot -10 :r 10 :b 20}
           (drawArc 0,0,100,30,0,360) shape2)
      (gfn shape2 1 {:rot 12 :sx 0.997 :sy 0.997 :x 2 :y 2 :a -10}
           (fillArc 0,0,100,30,0,90)
           shape1)
      (gfn shape2 0.05 {:sx 0.975 :sy 0.997 :rot 12 :x 1 :y 1 :shx 0.1 :branch 'shape1  }
           (drawArc 0,0,100,30,0,45)
           shape1)
      (gfn shape2 0.02 {:sx 0.975 :sy 0.995 :rot -10 :x 2 :y 2 :branch 'shape1 :g 50 }
           (drawArc 0,0,100,30,0,180)
           shape1))
)
(COMMENT

(cfam {:title "arcs2" :max 10000 :x 800 :y 700 :startshape 'shape0
      :branchdepth 2 :bmax 100}
      (gfn shape0 1 {:x 400 :y 350 :rot 45 :sx 2 :sy 2}
           (drawRect 0,0,0,0)
           shape1)      
      (gfn shape1 1 {:x 1 :y 1 :rot -10 :r 10 :b 20}
           (drawArc 0,0,100,30,0,360) shape2)
      (gfn shape2 1 {:rot 20 :sx 0.997 :sy 0.997 :x 1 :y 1 :a -10}
           (drawArc 0,0,100,30,-90,90)
           shape1)
      (gfn shape2 0.05 {:sx 0.975 :sy 0.997 :rot 12 :x 1 :y 1 :shx 0.1 :branch 'shape1  }
           (drawArc 0,0,100,30,0,45)
           shape1)
      (gfn shape2 0.02 {:sx 0.975 :sy 0.995 :rot -10 :x 1 :y 1 :branch 'shape1 :g 50 }
           (drawArc 0,0,100,30,0,180)
           shape1))
)
(COMMENT

(cfam {:title "arcs3" :max 1000 :x 800 :y 700 :startshape 'shape0
      :branchdepth 5 :bmax 50}
      (gfn shape0 1 {:x 400 :y 350 :rot 45 :sx 2 :sy 2}
           (drawRect 0,0,0,0)
           shape1)      
      (gfn shape1 1 {:x 1 :y 1 :rot -10 :r 10 :b 20}
           (drawArc 0,0,100,30,0,360) shape2)
      (gfn shape2 1 {:rot 20 :sx 0.997 :sy 0.997 :x 1 :y 1 :a -10}
           (drawArc 0,0,100,30,-90,90)
           shape1)
      (gfn shape2 0.05 {:sx 0.975 :sy 0.997 :rot 12 :x 1 :y 1 :shx 0.1 :branch 'shape1  }
           (drawArc 0,0,100,30,0,45)
           shape1)
      (gfn shape2 0.02 {:sx 0.975 :sy 0.995 :rot -10 :x 1 :y 1 :branch 'shape1 :g 50 }
           (drawArc 0,0,100,30,0,180)
           shape1))
)
(COMMENT

(cfam {:title "arcs4" :max 10000 :x 800 :y 700 :startshape 'shape0}
      (gfn shape0 1 {:x 400 :y 350 :sx 1.9 :sy 1.9}
           (drawRect 0,0,0,0)
           shape1)      
      (gfn shape1 1 {:x 0.02 :y 0.02 :rot -5 :sx 0.997 :sy 0.997 :r 10 :b 20}
           (drawArc 0,0,100,30,0,310) shape1))
)

(COMMENT

(cfam {:title "green-blue plant " :max 1000 :x 800 :y 700 :startshape 'shape0
      :branchdepth 3 :bmax 300}
      (gfn shape0 1 {:x 400 :y 550 :sx 2 :sy 2 :rot -90 :b 40}
           (drawRect 0,0,0,0)
           shape1)      
 
      (gfn shape1 1 {:x 0.1 :y 0.1 }
           (fillOval 0,0,1,1) shape1)
      (gfn shape1 0.1 {:x 0.1 :y 0.1 :g 1}
           (fillOval 0,0,1,1) shape1)
      (gfn shape1 0.08 {:x 0.1 :y 0.1 :rot 2  }
           (fillOval 0,0,1,1) shape1)
      (gfn shape1 0.08 {:x 0.1 :y 0.1 :rot -2   }
           (fillOval 0,0,1,1) shape1)
      (gfn shape1 0.005 {:x 0.1 :y 0.1 :rot -10  :g 2 :branch 'shape1}
           (fillOval 0,0,1,1) shape1)
      (gfn shape1 0.005 {:x 0.1 :y 0.1 :rot 10  :g 2 :branch 'shape1}
           (fillOval 0,0,1,1) shape1))
)
(COMMENT
 
(cfam {:title "green-blue bough" :max 1000 :x 800 :y 700 :startshape 'shape0
       :branchdepth 2 :bmax 500}
       (gfn shape0 1 {:x 200 :y 550 :sx 7 :sy 7 :rot -90 :b 40}
            (drawRect 0,0,0,0)
            shape1)      
       
       (gfn shape1 1 {:x 0.1 :y 0.1 :sx 0.998 :sy 0.998}
            (fillOval 0,0,3,1) shape1)
       (gfn shape1 0.1 {:x 0.1 :y 0.1 :g 1 :sx 0.998 :sy 0.998}
            (fillOval 0,0,3,1) shape1)
       (gfn shape1 0.08 {:x 0.1 :y 0.1 :rot 2  :sx 0.998 :sy 0.998}
            (fillOval 0,0,1,1) shape1)
       (gfn shape1 0.08 {:x 0.1 :y 0.1 :rot -2   :sx 0.998 :sy 0.998}
            (fillOval 0,0,1,1) shape1)
       (gfn shape1 0.005 {:x 0.1 :y 0.1 :rot -10  :g 2 :sx 0.99 :sy 0.99 :branch 'shape1}
            (fillOval 0,0,1,1) shape1)
       (gfn shape1 0.005 {:x 0.1 :y 0.1 :rot 10  :g 2 :sx 0.99 :sy 0.99  :branch 'shape1}
            (fillOval 0,0,1,1) shape1))
)
(COMMENT

(cfam {:title "hello world" :max 500 :x 800 :y 700 :startshape 'shape0
      :branchdepth 2 :bmax 10}
      (gfn shape0 1 {:x 2 :y 2 }
           (drawRect 0,0,0,0)
           shape1)      
      (gfn shape1 1 {:x 15 :y 12 :rot 30 :r 10 :b 20}
           (drawString "hello", 0,0) shape2)
      (gfn shape2 1.1 {:rot -29 :sx 0.99 :sy 0.99 :x 15 :y 15 :a -10 }
           (drawString "world", 5,2)
           shape1)
;(gfn shape2 0.1 {:rot -90 :sx 0.99 :sy 0.99 :x 5 :y 5 :a -10 :branch 'shape1}
;                 (drawString "world", 5,2)
;                                     shape1)

)
)

(COMMENT

(cfam {:title "line circles" :max 5000 :x 800 :y 700 :startshape 'shape0}
      (gfn shape0 1 {:x 100 :y 150 :rot 0}
           (drawRect 0,0,0,0)
           shape1)      
      (gfn shape1 1 {:rot 10 :x 5}
           (drawLine 0,0,100,100) shape1)
      (gfn shape1 0.07 {}
           (drawLine 0,0,0,0) shape2)
      (gfn shape2 1 {:rot -10 :x 2 }
           (drawLine 0,0,100,100) shape2)
      (gfn shape2 0.07 { :r 5 :b -2 }
           (drawLine 0,0,0,0) shape1))
)
(COMMENT

(cfam {:title "line circles" :max 50000 :x 800 :y 700 :startshape 'shape0}
      (gfn shape0 1 {:x 100 :y 150 :rot 0}
           (drawRect 0,0,0,0)
           shape1)      
      (gfn shape1 1 {:rot 10 :x 5}
           (drawLine 0,0,100,100) shape1)
      (gfn shape1 0.07 {}
           (drawLine 0,0,0,0) shape2)
      (gfn shape2 1 {:rot -10 :x 2 }
           (drawLine 0,0,100,100) shape2)
      (gfn shape2 0.07 { :r 5 :b -2 }
           (drawLine 0,0,0,0) shape1)
      (gfn shape2 0.03 { :sx 0.97 :sy 0.97 }
           (drawLine 0,0,0,0) shape3)
      (gfn shape3 1 { :x 80 :y 80 :rot 30}
                                        ;(drawArc -150,-150,300,300,0,90)
           (drawRect 0,0,0,0)
           shape1))
)
(COMMENT
(cfam {:title "hello world" :max 100 :x 800 :y 700 :startshape 'shape0
      :branchdepth 5 :bmax 20}

      (gfn shape0 1 {:x 100 :y 100 }
           (setFont (new java.awt.Font "Dialog", (. java.awt.Font PLAIN) 40))
           (drawRect 0,0,0,0)
           shape1)      
      (gfn shape1 1 {:x 25 :y 30 :rot 30 :r 10 :b 20}
           (drawString "hello", 0,0) shape2)
      (gfn shape2 1.1 {:rot -29 :sx 0.92 :sy 0.92 :x 35 :y 15 :a -1 }
           (drawString "world", 5,2)
                                     shape1)
      (gfn shape2 0.3 {:rot 20 :sx 0.92 :sy 0.92 :x 35 :y 15 :a -1 :branch 'shape1}
           (drawString "world", 5,2)
                                     shape1)
      (gfn shape2 0.1 {:rot 20 :sy -1 :x 35 :y 15 :a -1 :branch 'shape1}
           (drawString "world", 5,2)
           shape1))
)

(cfam {:title "red-black" :max 1000 :x 800 :y 700 :startshape 'shape0
      :branchdepth 3 :bmax 300}
      (gfn shape0 1 {:x 400 :y 550 :sx 2 :sy 2 :rot -90}
           (drawRect 0,0,0,0)
           shape1)      
      
      (gfn shape1 1 {:x 0.1 :y 0.1  :r 1}
           (fillOval 0,0,1,1) shape1)
      (gfn shape1 0.08 {:x 0.1 :y 0.1 :rot 2  :r 1}
           (fillOval 0,0,1,1) shape1)
      (gfn shape1 0.08 {:x 0.1 :y 0.1 :rot -2  :r 1 }
           (fillOval 0,0,1,1) shape1)
      (gfn shape1 0.08 {:x 0.1 :y 0.1 :rot -2  :r 1 }
           (fillOval 0,0,1,1) shape1)
      (gfn shape1 0.01 {:x 0.1 :y 0.1 :rot -10  :r 1 :branch 'shape1}
           (fillOval 0,0,1,1) shape1)
      (gfn shape1 0.01 {:x 0.1 :y 0.1 :rot 10  :r 1 :branch 'shape1}
           (fillOval 0,0,1,1) shape1))